@@ -1,15 +1,17 @@
+import json
 import os, re, sys, subprocess, platform
 import tarfile
-from distutils import log, version
+from distutils import log
 from contextlib import closing, contextmanager
 from ftplib import FTP
 
 try:
-    from urlparse import urljoin, unquote, urlparse
-    from urllib import urlretrieve, urlopen, urlcleanup
-except ImportError:
     from urllib.parse import urljoin, unquote, urlparse
-    from urllib.request import urlretrieve, urlopen, urlcleanup
+    from urllib.request import urlretrieve, urlopen, urlcleanup, Request
+except ImportError:  # Py2
+    from urlparse import urljoin, unquote, urlparse
+    from urllib import urlretrieve, urlcleanup
+    from urllib2 import urlopen, Request
 
 multi_make_options = []
 try:
@@ -23,20 +25,28 @@ except:
     pass
 
 
+# overridable to control script usage
+sys_platform = sys.platform
+
+
 # use pre-built libraries on Windows
 
 def download_and_extract_windows_binaries(destdir):
-    url = "https://github.com/lxml/libxml2-win-binaries/releases"
-    filenames = list(_list_dir_urllib(url))
+    url = "https://api.github.com/repos/lxml/libxml2-win-binaries/releases?per_page=5"
+    releases, _ = read_url(
+        url,
+        accept="application/vnd.github+json",
+        as_json=True,
+        github_api_token=os.environ.get("GITHUB_API_TOKEN"),
+    )
 
-    release_path = "/download/%s/" % find_max_version(
-        "library release", filenames, re.compile(r"/releases/tag/([0-9.]+[0-9])$"))
-    url += release_path
-    filenames = [
-        filename.rsplit('/', 1)[1]
-        for filename in filenames
-        if release_path in filename
-    ]
+    max_release = {'tag_name': ''}
+    for release in releases:
+        if max_release['tag_name'] < release.get('tag_name', ''):
+            max_release = release
+
+    url = "https://github.com/lxml/libxml2-win-binaries/releases/download/%s/" % max_release['tag_name']
+    filenames = [asset['name'] for asset in max_release.get('assets', ())]
 
     # Check for native ARM64 build or the environment variable that is set by
     # Visual Studio for cross-compilation (same variable as setuptools uses)
@@ -109,7 +119,7 @@ def unpack_zipfile(zipfn, destdir):
 
 
 def get_prebuilt_libxml2xslt(download_dir, static_include_dirs, static_library_dirs):
-    assert sys.platform.startswith('win')
+    assert sys_platform.startswith('win')
     libs = download_and_extract_windows_binaries(download_dir)
     for libname, path in libs.items():
         i = os.path.join(path, 'include')
@@ -164,13 +174,28 @@ def _list_dir_ftplib(url):
     return parse_text_ftplist("\n".join(data))
 
 
-def _list_dir_urllib(url):
-    with closing(urlopen(url)) as res:
+def read_url(url, decode=True, accept=None, as_json=False, github_api_token=None):
+    headers = {'User-Agent': 'https://github.com/lxml/lxml'}
+    if accept:
+        headers['Accept'] = accept
+    if github_api_token:
+        headers['authorization'] = "Bearer " + github_api_token
+    request = Request(url, headers=headers)
+
+    with closing(urlopen(request)) as res:
         charset = _find_content_encoding(res)
         content_type = res.headers.get('Content-Type')
         data = res.read()
 
-    data = data.decode(charset)
+    if decode:
+        data = data.decode(charset)
+    if as_json:
+        data = json.loads(data)
+    return data, content_type
+
+
+def _list_dir_urllib(url):
+    data, content_type = read_url(url)
     if content_type and content_type.startswith('text/html'):
         files = parse_html_filelist(data)
     else:
@@ -178,26 +203,28 @@ def _list_dir_urllib(url):
     return files
 
 
-def http_find_latest_version_directory(url):
-    with closing(urlopen(url)) as res:
-        charset = _find_content_encoding(res)
-        data = res.read()
+def http_find_latest_version_directory(url, version=None):
+    data, _ = read_url(url)
     # e.g. <a href="1.0/">
     directories = [
         (int(v[0]), int(v[1]))
-        for v in re.findall(r' href=["\']([0-9]+)\.([0-9]+)/?["\']', data.decode(charset))
+        for v in re.findall(r' href=["\']([0-9]+)\.([0-9]+)/?["\']', data)
     ]
     if not directories:
         return url
-    latest_dir = "%s.%s" % max(directories)
+    best_version = max(directories)
+    if version:
+        major, minor, _ = version.split(".", 2)
+        major, minor = int(major), int(minor)
+        if (major, minor) in directories:
+            best_version = (major, minor)
+    latest_dir = "%s.%s" % best_version
     return urljoin(url, latest_dir) + "/"
 
 
 def http_listfiles(url, re_pattern):
-    with closing(urlopen(url)) as res:
-        charset = _find_content_encoding(res)
-        data = res.read()
-    files = re.findall(re_pattern, data.decode(charset))
+    data, _ = read_url(url)
+    files = re.findall(re_pattern, data)
     return files
 
 
@@ -248,7 +275,7 @@ def download_libxml2(dest_dir, version=None):
         from_location = "https://gitlab.gnome.org/GNOME/libxml2/-/archive/dea91c97debeac7c1aaf9c19f79029809e23a353/"
         version = "dea91c97debeac7c1aaf9c19f79029809e23a353"
     else:
-        from_location = http_find_latest_version_directory(LIBXML2_LOCATION)
+        from_location = http_find_latest_version_directory(LIBXML2_LOCATION, version=version)
 
     return download_library(dest_dir, from_location, 'libxml2',
                             version_re, filename, version=version)
@@ -259,7 +286,7 @@ def download_libxslt(dest_dir, version=None):
     #version_re = re.compile(r'LATEST_LIBXSLT_IS_([0-9.]+[0-9](?:-[abrc0-9]+)?)')
     version_re = re.compile(r'libxslt-([0-9.]+[0-9]).tar.xz')
     filename = 'libxslt-%s.tar.xz'
-    from_location = http_find_latest_version_directory(LIBXSLT_LOCATION)
+    from_location = http_find_latest_version_directory(LIBXSLT_LOCATION, version=version)
     return download_library(dest_dir, from_location, 'libxslt',
                             version_re, filename, version=version)
 
@@ -288,7 +315,7 @@ def find_max_version(libname, filenames, version_re=None):
         match = version_re.search(fn)
         if match:
             version_string = match.group(1)
-            versions.append((tuple(map(tryint, version_string.split('.'))),
+            versions.append((tuple(map(tryint, version_string.replace("-", ".-").split('.'))),
                              version_string))
     if not versions:
         raise Exception(
@@ -393,13 +420,13 @@ def cmmi(configure_cmd, build_dir, multicore=None, **call_setup):
 
 def configure_darwin_env(env_setup):
     import platform
-    # configure target architectures on MacOS-X (x86_64 only, by default)
+    # configure target architectures on MacOS-X (x86_64 + Arm64, by default)
     major_version, minor_version = tuple(map(int, platform.mac_ver()[0].split('.')[:2]))
-    if major_version > 7:
+    if major_version >= 11:
         env_default = {
-            'CFLAGS': "-arch x86_64 -O2",
-            'LDFLAGS': "-arch x86_64",
-            'MACOSX_DEPLOYMENT_TARGET': "10.6"
+            'CFLAGS': "-arch x86_64 -arch arm64 -O3",
+            'LDFLAGS': "-arch x86_64 -arch arm64",
+            'MACOSX_DEPLOYMENT_TARGET': "11.0"
         }
         env_default.update(os.environ)
         env_setup['env'] = env_default
@@ -444,7 +471,7 @@ def build_libxml2xslt(download_dir, build_dir,
         return found
 
     call_setup = {}
-    if sys.platform == 'darwin':
+    if sys_platform == 'darwin':
         configure_darwin_env(call_setup)
 
     configure_cmd = ['./configure',
@@ -525,3 +552,27 @@ def build_libxml2xslt(download_dir, build_dir,
         if lib in filename and filename.endswith('.a')]
 
     return xml2_config, xslt_config
+
+
+def main():
+    static_include_dirs = []
+    static_library_dirs = []
+    download_dir = "libs"
+
+    if sys_platform.startswith('win'):
+        return get_prebuilt_libxml2xslt(
+            download_dir, static_include_dirs, static_library_dirs)
+    else:
+        return build_libxml2xslt(
+            download_dir, 'build/tmp',
+            static_include_dirs, static_library_dirs,
+            static_cflags=[],
+            static_binaries=[]
+        )
+
+
+if __name__ == '__main__':
+    if len(sys.argv) > 1:
+        # change global sys_platform setting
+        sys_platform = sys.argv[1]
+    main()
