@@ -271,6 +271,26 @@ class CleanerTest(unittest.TestCase):
             expected,
             cleaner.clean_html(html))
 
+    def test_host_whitelist_valid(self):
+        # Frame with valid hostname in src is allowed.
+        html = '<div><iframe src="https://example.com/page"></div>'
+        expected = '<div><iframe src="https://example.com/page"></iframe></div>'
+        cleaner = Cleaner(frames=False, host_whitelist=["example.com"])
+        self.assertEqual(expected, cleaner.clean_html(html))
+
+    def test_host_whitelist_invalid(self):
+        html = '<div><iframe src="https://evil.com/page"></div>'
+        expected = '<div></div>'
+        cleaner = Cleaner(frames=False, host_whitelist=["example.com"])
+        self.assertEqual(expected, cleaner.clean_html(html))
+
+    def test_host_whitelist_sneaky_userinfo(self):
+        # Regression test: Don't be fooled by hostname and colon in userinfo.
+        html = '<div><iframe src="https://example.com:@evil.com/page"></div>'
+        expected = '<div></div>'
+        cleaner = Cleaner(frames=False, host_whitelist=["example.com"])
+        self.assertEqual(expected, cleaner.clean_html(html))
+
 
 def test_suite():
     suite = unittest.TestSuite()
