@@ -8,7 +8,7 @@ from __future__ import absolute_import
 
 import unittest
 
-from .common_imports import etree, BytesIO, HelperTestCase, fileInTestDir, make_doctest
+from .common_imports import etree, BytesIO, HelperTestCase, fileInTestDir, make_doctest, SimpleFSPath
 
 
 class ETreeXMLSchemaTestCase(HelperTestCase):
@@ -66,8 +66,10 @@ class ETreeXMLSchemaTestCase(HelperTestCase):
         for a _LogEntry object (or even a node for which to determine
         a path), but at least when this test was created schema validation
         errors always got a node and an XPath value. If that ever changes,
-        we can modify this test to something like:
+        we can modify this test to something like::
+
             self.assertTrue(error_path is None or tree_path == error_path)
+
         That way, we can at least verify that if we did get a path value
         it wasn't bogus.
         """
@@ -385,6 +387,11 @@ class ETreeXMLSchemaTestCase(HelperTestCase):
         etree.XMLSchema(schema_element)
         etree.XMLSchema(schema_element)
 
+    def test_xmlschema_pathlike(self):
+        schema = etree.XMLSchema(file=SimpleFSPath(fileInTestDir('test.xsd')))
+        tree_valid = self.parse('<a><b></b></a>')
+        self.assertTrue(schema.validate(tree_valid))
+
 
 class ETreeXMLSchemaResolversTestCase(HelperTestCase):
     resolver_schema_int = BytesIO("""\
